@@ -36,7 +36,7 @@
 #define LINUX_2_2
 #endif
 
-#ifdef ALSA_BUILD
+#if defined(ALSA_BUILD) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 #if defined(CONFIG_MODVERSIONS) && !defined(__GENKSYMS__) && !defined(__DEPEND__)
 #define MODVERSIONS
 #include <linux/modversions.h>
@@ -46,7 +46,7 @@
 #undef MODVERSIONS
 #undef _set_ver
 #endif
-#endif /* ALSA_BUILD */
+#endif /* ALSA_BUILD && KERNEL < 2.6.0 */
 
 #include <linux/module.h>
 
@@ -127,13 +127,23 @@ void snd_compat_request_module(const char *name, ...);
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 3) && !defined(need_resched)
 #define need_resched() (current->need_resched)
 #endif
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 4) && !defined(WE_ARE_WOLK)
+#ifndef CONFIG_HAVE_PDE
 #include <linux/fs.h>
+#include <linux/proc_fs.h>
 static inline struct proc_dir_entry *PDE(const struct inode *inode)
 {
 	return (struct proc_dir_entry *) inode->u.generic_ip;
 }
 #endif
+#ifndef cond_resched
+#define cond_resched() \
+	do { \
+		if (need_resched()) { \
+			set_current_state(TASK_RUNNING); \
+			schedule(); \
+		} \
+	} while (0)
+#endif
 #include <asm/io.h>
 #if !defined(isa_virt_to_bus)
 #if defined(virt_to_bus) || defined(__alpha__)
@@ -325,6 +335,11 @@ int snd_hack_usb_set_interface(struct usb_device *dev, int interface, int altern
 #define get_endpoint(alt,ep)	(&(alt)->endpoint[ep])
 #define get_ep_desc(ep)		(ep)
 #define get_cfg_desc(cfg)	(cfg)
+#define USB_DT_CS_DEVICE	0x21
+#define USB_DT_CS_CONFIG	0x22
+#define USB_DT_CS_STRING	0x23
+#define USB_DT_CS_INTERFACE	0x24
+#define USB_DT_CS_ENDPOINT	0x25
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 45)
@@ -395,6 +410,16 @@ static inline void *snd_compat_vmap(struct page **pages, unsigned int count, uns
 #define EXPORT_NO_SYMBOLS
 #endif
 
+/* MODULE_ALIAS & co. */
+#ifndef MODULE_ALIAS
+#define MODULE_ALIAS(x)
+#define MODULE_ALIAS_CHARDEV_MAJOR(x)
+#endif
+
+#ifndef CONFIG_HAVE_PCI_CONSISTENT_DMA_MASK
+#define pci_set_consistent_dma_mask(p,x) pci_set_dma_mask(p,x)
+#endif
+
 #include "amagic.h"
 
 #endif /* __SOUND_LOCAL_DRIVER_H */
